# coding: utf-8

"""
    Fabric Orchestrator API

    This is Fabric Orchestrator API  # noqa: E501

    OpenAPI spec version: 1.0.1
    Contact: kthare10@unc.edu
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import unittest

from fabric_cf.orchestrator.swagger_client.models.slice import Slice  # noqa: E501
from fabric_cf.orchestrator.swagger_client.rest import ApiException


class TestSlice(unittest.TestCase):
    """Slice unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testSlice(self):
        """Test Slice"""
        # FIXME: construct object with mandatory attributes with example values
        # model = fabric_cf.orchestrator.swagger_client.models.slice.Slice()  # noqa: E501
        pass


if __name__ == '__main__':
    unittest.main()
