# coding: utf-8

"""
    Fabric Orchestrator API

    This is Fabric Orchestrator API  # noqa: E501

    OpenAPI spec version: 1.0.1
    Contact: kthare10@unc.edu
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class PoaData(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'operation': 'str',
        'poa_id': 'str',
        'state': 'str',
        'sliver_id': 'str',
        'slice_id': 'str',
        'error': 'str',
        'info': 'object'
    }

    attribute_map = {
        'operation': 'operation',
        'poa_id': 'poa_id',
        'state': 'state',
        'sliver_id': 'sliver_id',
        'slice_id': 'slice_id',
        'error': 'error',
        'info': 'info'
    }

    def __init__(self, operation=None, poa_id=None, state=None, sliver_id=None, slice_id=None, error=None, info=None):  # noqa: E501
        """PoaData - a model defined in Swagger"""  # noqa: E501
        self._operation = None
        self._poa_id = None
        self._state = None
        self._sliver_id = None
        self._slice_id = None
        self._error = None
        self._info = None
        self.discriminator = None
        if operation is not None:
            self.operation = operation
        if poa_id is not None:
            self.poa_id = poa_id
        if state is not None:
            self.state = state
        if sliver_id is not None:
            self.sliver_id = sliver_id
        if slice_id is not None:
            self.slice_id = slice_id
        if error is not None:
            self.error = error
        if info is not None:
            self.info = info

    @property
    def operation(self):
        """Gets the operation of this PoaData.  # noqa: E501


        :return: The operation of this PoaData.  # noqa: E501
        :rtype: str
        """
        return self._operation

    @operation.setter
    def operation(self, operation):
        """Sets the operation of this PoaData.


        :param operation: The operation of this PoaData.  # noqa: E501
        :type: str
        """

        self._operation = operation

    @property
    def poa_id(self):
        """Gets the poa_id of this PoaData.  # noqa: E501


        :return: The poa_id of this PoaData.  # noqa: E501
        :rtype: str
        """
        return self._poa_id

    @poa_id.setter
    def poa_id(self, poa_id):
        """Sets the poa_id of this PoaData.


        :param poa_id: The poa_id of this PoaData.  # noqa: E501
        :type: str
        """

        self._poa_id = poa_id

    @property
    def state(self):
        """Gets the state of this PoaData.  # noqa: E501


        :return: The state of this PoaData.  # noqa: E501
        :rtype: str
        """
        return self._state

    @state.setter
    def state(self, state):
        """Sets the state of this PoaData.


        :param state: The state of this PoaData.  # noqa: E501
        :type: str
        """

        self._state = state

    @property
    def sliver_id(self):
        """Gets the sliver_id of this PoaData.  # noqa: E501


        :return: The sliver_id of this PoaData.  # noqa: E501
        :rtype: str
        """
        return self._sliver_id

    @sliver_id.setter
    def sliver_id(self, sliver_id):
        """Sets the sliver_id of this PoaData.


        :param sliver_id: The sliver_id of this PoaData.  # noqa: E501
        :type: str
        """

        self._sliver_id = sliver_id

    @property
    def slice_id(self):
        """Gets the slice_id of this PoaData.  # noqa: E501


        :return: The slice_id of this PoaData.  # noqa: E501
        :rtype: str
        """
        return self._slice_id

    @slice_id.setter
    def slice_id(self, slice_id):
        """Sets the slice_id of this PoaData.


        :param slice_id: The slice_id of this PoaData.  # noqa: E501
        :type: str
        """

        self._slice_id = slice_id

    @property
    def error(self):
        """Gets the error of this PoaData.  # noqa: E501


        :return: The error of this PoaData.  # noqa: E501
        :rtype: str
        """
        return self._error

    @error.setter
    def error(self, error):
        """Sets the error of this PoaData.


        :param error: The error of this PoaData.  # noqa: E501
        :type: str
        """

        self._error = error

    @property
    def info(self):
        """Gets the info of this PoaData.  # noqa: E501


        :return: The info of this PoaData.  # noqa: E501
        :rtype: object
        """
        return self._info

    @info.setter
    def info(self, info):
        """Sets the info of this PoaData.


        :param info: The info of this PoaData.  # noqa: E501
        :type: object
        """

        self._info = info

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(PoaData, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, PoaData):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other
